package Network.Algorithm;


import Network.Algorithm.FlowAugmentingDecreasingPathsAlgorithm.*;
import Network.Algorithm.SimplexAlgorithm.*;
import Network.Algorithm.TreeAlgorithm.*;



public class ArcAlgInfoFactory
{

    private ArcAlgInfoStack flow;    //FlowAugmentingDecreasingPathsAlgArcInfo only
    private ArcAlgInfoStack simplex; //SimplexAlgArcInfo only
    private ArcAlgInfoStack tree;    //TreeArcInfo only


    public ArcAlgInfoFactory(int initialSize_, int capacityIncrement_)
    {
        flow = new ArcAlgInfoStack(initialSize_, capacityIncrement_);
        simplex = new ArcAlgInfoStack(initialSize_, capacityIncrement_);
        tree = new ArcAlgInfoStack(initialSize_, capacityIncrement_);
    }


    public final FlowAugmentingDecreasingPathsAlgArcInfo makeFlowInfo()
    {
        FlowAugmentingDecreasingPathsAlgArcInfo info =
            (FlowAugmentingDecreasingPathsAlgArcInfo)flow.pop();
        if (info != null)
            return info;
        else
            return new FlowAugmentingDecreasingPathsAlgArcInfo();
    }

    public final SimplexAlgArcInfo makeSimplexInfo()
    {
        SimplexAlgArcInfo info =
            (SimplexAlgArcInfo)simplex.pop();
        if (info != null)
            return info;
        else
            return new SimplexAlgArcInfo();
    }

    public final TreeArcInfo makeTreeInfo()
    {
        TreeArcInfo info =
            (TreeArcInfo)tree.pop();
        if (info != null)
            return info;
        else
            return new TreeArcInfo();
    }

    public void collectInfo(ArcAlgInfo info)
    {
        if (info instanceof TreeArcInfo)
            tree.push(info);
        else if (info instanceof SimplexAlgArcInfo)
            simplex.push(info);
        else if (info instanceof FlowAugmentingDecreasingPathsAlgArcInfo)
            flow.push(info);
    }

}


final class ArcAlgInfoStack
{

    private ArcAlgInfo[] stack;    //null not a valid entry
    private int size = 0;          //Number of valid entries on stack
    private int capacityIncrement; //Amount by which to increase stack length when we resize


    ArcAlgInfoStack(int initialSize_, int capacityIncrement_)
    {
        stack = new ArcAlgInfo[initialSize_];
        if (capacityIncrement_ < 1)
            throw new IllegalArgumentException("ArcAlgInfoStack: capacityIncrement_ must be >= 1.");
        else
            capacityIncrement = capacityIncrement_;
    }


    synchronized void push(ArcAlgInfo info)
    {
        if (size == stack.length)
            resize();
        stack[size++] = info;
    }

    //Returns null if empty
    synchronized ArcAlgInfo pop()
    {
        if (size > 0)
            return stack[--size];
        else
            return null;
    }

    int getSize()
    {
        return (size);
    }

    private synchronized void resize()
    {
        ArcAlgInfo[] resized = new ArcAlgInfo[size + capacityIncrement];
        System.arraycopy(stack, 0, resized, 0, size);
        stack = resized;
    }

}

