package Network.Representation;



public final class NodeList {

    public Node elem;
    
    public NodeList next;
    
    public NodeList prev;


    public NodeList(Node elem_, NodeList next_, NodeList prev_)
    {
        elem = elem_;
        next = next_;
        prev = prev_;
    }

    public NodeList(Node elem_)
    {
        elem = elem_;
        next = prev =null;
    }


    public final void initialize(Node elem_, NodeList next_, NodeList prev_)
    {
        elem = elem_;
        next = next_;
        prev = prev_;
    }
    
    public final void initialize(Node elem_)
    {
        elem = elem_;
        next = prev = null;
    }


    public final void addAfter(NodeList nodeList)
    {
        NodeList temp = next;
        next = nodeList;
        nodeList.prev = this;
        nodeList.next = temp;
        if (temp != null)
            temp.prev = nodeList;
    }
    
    public final void addBefore(NodeList nodeList)
    {
        NodeList temp = prev;
        prev = nodeList;
        nodeList.next = this;
        nodeList.prev = temp;
        if (temp != null)
            temp.next = nodeList;
    }
    
    public final void remove()
    {
        if (prev != null)
            prev.next = next;
        if (next != null)
            next.prev = prev;
    }
    
    public final NodeList find(Node elem_)
    {
        NodeList nextNodeList = this;
        while (nextNodeList != null) {
            if (nextNodeList.elem.equals(elem_))
                return (nextNodeList);
            nextNodeList = nextNodeList.next;
        }
        return (null);
    }
    
    public final boolean contains(Node elem_)
    {
        return (find(elem_) != null);
    }
    
}


final class NodeListStack {

    private NodeList[] stack;
    private int size = 0;
    private int capacity;
    private int capacityIncrement;
    
    
    NodeListStack(int capacity_, int capacityIncrement_)
    {
        capacity = capacity_;
        capacityIncrement = capacityIncrement_;
        stack = new NodeList[capacity];
    }
    
    
    final synchronized void push(NodeList nodeList)
    {
        if (size >= capacity)
            resize();
        stack[size++] = nodeList;
    }
    
    final synchronized NodeList pop()
    {
        if (size > 0)
            return (stack[--size]);
        else
            return (null);
    }
    
    final synchronized private void resize()
    {
        NodeList[] newStack = new NodeList[capacity + capacityIncrement];
        System.arraycopy(stack, 0, newStack, 0, stack.length - 1);
        stack = newStack;
        capacity += capacityIncrement;
    }

}
